@@ -251,6 +251,16 @@ bool file::set_modification_time(datetime const& t)
 	return SetFileTime(fd_, nullptr, &ft, &ft) == TRUE;
 }
 
+datetime file::get_modification_time()
+{
+	FILETIME ft{};
+
+	if (GetFileTime(fd_, nullptr, nullptr, &ft)) {
+		return datetime(ft, datetime::milliseconds);
+	};
+
+	return {};
+}
 #else
 
 file::file(file && op) noexcept
@@ -299,6 +309,8 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		case ENOENT:
 			return {result::nofile, err};
 		case EACCES:
+		case EPERM:
+		case EROFS:
 			return {result::noperm, err};
 		case EDQUOT:
 		case ENOSPC:
@@ -453,6 +465,7 @@ result remove_file(native_string const& name, bool missing_file_is_error)
 		return {result::nofile, err};
 	case EACCES:
 	case EPERM:
+	case EROFS:
 		return {result::noperm, err};
 	case EINVAL:
 	case ENAMETOOLONG:
@@ -483,6 +496,21 @@ bool file::set_modification_time(datetime const& t)
 	times[1].tv_nsec = t.get_milliseconds() * 1000000;
 	return futimens(fd_, times) == 0;
 }
+
+datetime file::get_modification_time()
+{
+	struct stat buf;
+
+	if (fstat(fd_, &buf) == 0) {
+#if HAVE_STRUCT_STAT_ST_MTIM
+		return datetime(buf.st_mtim.tv_sec, datetime::milliseconds) + fz::duration::from_milliseconds(buf.st_mtim.tv_nsec/1000000);
+#else
+		return datetime(buf.st_mtime, datetime::seconds);
+#endif
+	}
+
+	return {};
+}
 #endif
 
 rwresult read_file(fz::file & f, buffer & out, size_t max_size)
